/**
 * SControl: 
 * ---------
 * steuert den Prozess einer Suchanfrage, indem die einzelnen Komponenten 
 * der Suchmaschine (FileReader, Parser und Analyzer) der Reihe nach aufgerufen werden.
 */

import java.util.*;
import java.io.File;

public class SControl
{
    // Klassenvariablen
    protected static Scanner input = new Scanner(System.in); // Neues 'Scanner'-Objekt erzeugen und der Variable 'input' zuweisen

    // Instanzvariablen
    private FileReader myReader;
    private Parser myParser;
    private Analyzer myAnalyzer;
    private String path = "C:/Temp";
    private String suchPattern;

    /**
     * Konstruktor
     */
    public SControl()
    {
        // Komponenten der Suchmaschine initialisieren
        myReader = new FileReader();
        myParser = new Parser();
        myAnalyzer = new Analyzer();
        newSearch();
    }

    /**
     * newSearch():
     * durchsucht das Verzeichnis 'path', indem für jede Datei in 'path' 
     * die Methode searchFile() aufgerufen wird. 
     */
    public void newSearch()
    {
        // Initialisierung
        String eingabe = "";
        suchPattern ="";
        
        // Zu durchsuchendes Verzeichnis bestimmen
        System.out.println("Aktuelles Verzeichnis: " + path);
        System.out.println("Neues Verzeichnis [j/n]?");
        while ((!eingabe.equals("j")) && (!eingabe.equals("n"))) {
           eingabe = input.nextLine();
        }
        if (eingabe.equals("j")){
           System.out.println();
           System.out.println("Bitte geben Sie das zu durchsuchende Verzeichnis ein: ");
           path = input.nextLine();
        }
        
        // Suchstring einlesen
        System.out.println();
        System.out.println("Geben Sie den Suchstring ein: ");
        while (suchPattern.equals("")){
           suchPattern = input.nextLine();
        }
        System.out.println();
        
        // Dateien des Verzeichnis 'path' einlesen und durchsuchen
        File dir = new File(path);
        File[] files = dir.listFiles();
	    if (files != null) {
		   for (int i = 0; i < files.length; i++) {
			  if (!files[i].isDirectory()) {  // wenn die Datei kein Verzeichnis ist
				 searchFile(files[i]);
			  }
		   }
        } else {
            System.out.println("ACHTUNG: Das Verzeichnis \"" + dir.getName() + "\" ist entweder leer oder existiert nicht!");
        }  
    }
    
    /**
     * searchFile(File f):
     * -------------------
     * durchsucht eine HTML-Datei 'f' nach der Häufigkeit des Auftretens des 
     * Suchpatterns 'suchPattern' in den Abschnitten <title> und <body>.
     * Es werden Fehlermeldungen ausgegeben, wenn die Datei keine HTML-Datei ist, 
     * oder wenn Probleme beim Lesen der Datei auftraten.
     */
    private void searchFile(File f){
        /** Aufruf der Aufgabe von Team 1 */
        Quelltext content = myReader.fileToString(f);
        
        // Schönheitskorrektur (siehe unten)
        haircut(content.txt);  
        
        if (!content.fehler){ // Wenn kein Fehler aufgetreten ist
            /** Aufruf der Aufgabe von Team 2 */
            HtmlContent html = myParser.parse(content.txt); 
            
            // Schönheitskorrektur (siehe unten)
            haircut(html.title);  
            haircut(html.body);
            
            if (!html.keinHTML){ // Wenn die Datei eine HTML-Datei ist
                /** Aufruf der Aufgabe von Team 3 */
                Treffer count = myAnalyzer.trefferCount(html, suchPattern); 
               
                /** Ausgabe der Treffer: Ausgangspunkt für Aufgabe von Team 4 (optional) */ 
                System.out.println("Die Datei \"" + f.getName() + "\" enthält ...");
                System.out.println("...  " + count.titleMatches + " Übereinstimmungen im <title>");
                System.out.println("...  " + count.bodyMatches + " Übereinstimmungen im <body>");
                System.out.println();
               
                // Fehlerbehandlung ...
            } else { // ... wenn die gelesene Datei keine HTML-Datei ist ...
                System.out.println("ACHTUNG: Die Datei \"" + f.getName() + "\" ist keine HTML-Datei!");
                System.out.println();
            }
        } else { // ... wenn beim Lesen der Datei ein Fehler auftrat ... 
            System.out.println("ACHTUNG: Beim Lesen der Datei \"" + f.getName() + "\" ist ein Fehler aufgetreten!");
            System.out.println();
        }
    }
    
    /**
     * haircut(String s):
     * -------------------
     * Eine kleine Schönheitskorrektur für Zeichenketten.
     * Entfernt doppelte Leerzeichen sowie Sonderzeichen an Anfang und Ende des Strings.
     */
    private String haircut(String s){
        s = s.trim();
        while (s.indexOf("  ") >= 0){
           s = s.replaceAll("  ", " "); 
        }
        return s;
    }   
}
